/***********************************************************************************
* T-SQL: Simple Changes That Go a Long Way
* Dave Valentine
* Non-Clustered Indexes Demo SQL Script
***********************************************************************************/

USE AdventureWorks2012;
GO

--SET STATISTICS IO ON so we can see disk activity
SET STATISTICS IO ON;
GO



/***********************************************************************************
* Create Production.BackupTransactionHistory using the INTO statement.  Adding the 
* primary key constraint to the table as well.
***********************************************************************************/
-- Remove Production.BackupTransactionHistory if exists
IF OBJECT_ID('Production.BackupTransactionHistory') IS NOT NULL 
   DROP TABLE Production.BackupTransactionHistory;
GO


SELECT   TransactionID
       , ProductID
       , ReferenceOrderID
       , ReferenceOrderLineID
       , TransactionDate
       , TransactionType
       , Quantity
       , ActualCost
       , ModifiedDate
INTO     Production.BackupTransactionHistory
FROM     Production.TransactionHistory; 
GO


-- Create the PRIMARY KEY
ALTER TABLE Production.BackupTransactionHistory 
ADD CONSTRAINT PK_BackupTransactionHistory PRIMARY KEY (TransactionID) 
GO



/***********************************************************************************
* PART I - Covered Indexes
*
* Running the following query will result in an index scan.  Using Ctrl-M
* will display the Execution Plan that shows the Index Scan. 
***********************************************************************************/
SELECT   TransactionType
       , Quantity
       , ActualCost
FROM     Production.BackupTransactionHistory
WHERE    ProductID = 843;
GO



/***********************************************************************************
* By adding a NONCLUSTERED INDEX on ProductID to Production.BackupTransactionHistory 
* we can eliminate the index scan
***********************************************************************************/
CREATE NONCLUSTERED INDEX IX_BackupTransactionHistory_ProductID 
ON Production.BackupTransactionHistory(ProductID ASC);
GO



/***********************************************************************************
* Running the query again will result in an index seek.  However, we now have
* a key lookup.
***********************************************************************************/
SELECT   TransactionType
       , Quantity
       , ActualCost
FROM     Production.BackupTransactionHistory
WHERE    ProductID = 843;
GO



/***********************************************************************************
* Adding a covering index will elminate the key lookup.  Start by removing the old
* index, then we can creat the covering index.
***********************************************************************************/
--Drop the existing index on product id
DROP INDEX IX_BackupTransactionHistory_ProductID 
ON Production.BackupTransactionHistory;
GO


-- Create covering NONCLUSTERED INDEX on ProductID INCLUDE SELECT columns.
CREATE NONCLUSTERED INDEX IX_BackupTransactionHistory_ProductID 
ON Production.BackupTransactionHistory(ProductID ASC)
INCLUDE(TransactionType, Quantity, ActualCost);
GO



/***********************************************************************************
* Running the query again will result in an index seek without the key lookup.
***********************************************************************************/
SELECT   TransactionType
       , Quantity
       , ActualCost
FROM     Production.BackupTransactionHistory
WHERE    ProductID = 843;
GO



/***********************************************************************************
* PART II - Filtered Indexes
*
* Running the following query for a range of values greater than $2400.00 again
* yields an index scan and page reads.
***********************************************************************************/
SELECT   TransactionType
       , Quantity
       , ActualCost
FROM     Production.BackupTransactionHistory
WHERE    ActualCost > 2400;
GO



/***********************************************************************************
* Adding a covering NONCLUSTERED INDEX on ActualCost to 
* Production.BackupTransactionHistory will produce an index seek and reduce the 
* logical page reads.
***********************************************************************************/
CREATE NONCLUSTERED INDEX IX_BackupTransactionHistory_ActualCost 
ON Production.BackupTransactionHistory(ActualCost ASC)
INCLUDE(TransactionType, Quantity);
GO



/***********************************************************************************
* Running the query again shows that the covering NONCLUSTERED INDEX is being 
* used, and the page reads are significantly less.
***********************************************************************************/
SELECT   TransactionType
       , Quantity
       , ActualCost
FROM     Production.BackupTransactionHistory
WHERE    ActualCost > 2400;
GO



/***********************************************************************************
* Adding a filtered NONCLUSTERED INDEX on ActualCost to 
* Production.BackupTransactionHistory will reduces the index size and the 
* logical page reads.
***********************************************************************************/
CREATE NONCLUSTERED INDEX IX_BackupTransactionHistory_Filtered 
ON Production.BackupTransactionHistory(ActualCost ASC)
INCLUDE(TransactionType, Quantity)
WHERE ActualCost > 2000;
GO



/***********************************************************************************
* Running the query again shows that the filtered NONCLUSTERED INDEX is being 
* used, and the page reads are less.
***********************************************************************************/
SELECT   TransactionType
       , Quantity
       , ActualCost
FROM     Production.BackupTransactionHistory
WHERE    ActualCost > 2400;
GO



/***********************************************************************************
* This Script shows the index type, the key columns used by an index, the included
* columns for the index, and the index size in KB and MB.
*
* Running this script shows that we have three indexes on the 
* Production.BackupTransactionHistory table.  Notice the size difference between
* the different types of indexes.
***********************************************************************************/
DECLARE @TableName sysname = 'Production.BackupTransactionHistory';
SELECT   TableName = s.name + '.' + o.name
       , IndexName = i.name
       , IndexType = i.type_desc
       , IndexKeyColumns = SUBSTRING((
                                      SELECT ', ' + c.name
                                      FROM   sys.index_columns ic
                                      JOIN   sys.columns c ON c.column_id = ic.column_id
                                                              AND c.object_id = ic.object_id
                                      WHERE  ic.index_id = i.index_id
                                             AND ic.object_id = i.object_id
                                             AND ic.is_included_column = 0
                                      ORDER BY key_ordinal FOR XML PATH('')), 3, 1000)
       , IncludedColumns = SUBSTRING((
                                      SELECT ', ' + c.name
                                      FROM   sys.index_columns ic
                                      JOIN   sys.columns c ON c.column_id = ic.column_id
                                                              AND c.object_id = ic.object_id
                                      WHERE  ic.index_id = i.index_id
                                             AND ic.object_id = i.object_id
                                             AND ic.is_included_column = 1
                                      ORDER BY c.name FOR XML PATH('')), 3, 1000)
       , IndexSizeKB = CAST(ps.used_page_count * 8 AS int)
	   , IndexSizeMB = CAST(ps.used_page_count * 8 / 1024.0 AS decimal(9,2))	   
FROM     sys.objects o
JOIN     sys.schemas s ON s.schema_id = o.schema_id
JOIN     sys.indexes i ON i.object_id = o.object_id
JOIN     sys.dm_db_partition_stats ps ON ps.index_id = i.index_id
                                         AND ps.object_id = i.object_id
                                         AND ps.object_id = o.object_id
WHERE    o.object_id = OBJECT_ID(@TableName);
GO


/* END Non-Clustered Indexes DEMO */
