/***********************************************************************************
* T-SQL: Simple Changes That Go a Long Way
* Dave Valentine
* SARGable Demo SQL Script
***********************************************************************************/

USE AdventureWorks2012;
GO

-- SET STATISTICS IO ON so we can see disk activity
SET STATISTICS IO ON;
GO



/***********************************************************************************
* PART I - Type Confusion
* Youre a DBA Debugging a query for heavily used Reporting Services Report.  
* This report shows credit card details when a user enters a credit card number.  
* Users are claiming that the report is taking a long time to return results.  
* You determine that the issue with the report is the following query.   
************************************************************************************/
SELECT CreditCardID
     , CardType
     , ExpMonth
     , ExpYear
FROM Sales.CreditCard cc
WHERE CardNumber = 55552127249722;
GO


-- Create a Covering NONCLUSTERED INDEX   
CREATE NONCLUSTERED INDEX IX_CreditCard_CardNumber 
ON Sales.CreditCard(CardNumber)
INCLUDE(CardType, ExpMonth, ExpYear);
GO


-- Our Index is not being used becuase of Implicit Type Conversion
SELECT CreditCardID
     , CardType
     , ExpMonth
     , ExpYear
FROM Sales.CreditCard cc
WHERE CardNumber = 55552127249722;
GO


--Updating our predicate with the correct type and our index is used
SELECT CreditCardID
     , CardType
     , ExpMonth
     , ExpYear
FROM Sales.CreditCard cc
WHERE CardNumber = N'55552127249722';
GO



/***********************************************************************************
* PART II - Functions, Functions, Functions
* Youre a DBA Debugging a query for application that provides a wild card search 
* based on the first three characters of a product.  Users are reporting that the
* application slows down when using this feature.  You determine that the issue with 
* the application feature is the following query.
************************************************************************************/
DECLARE @Name nvarchar(50) = 'Mou';
SELECT Name
     , ProductNumber
     , Color
     , StandardCost
FROM Production.Product P
WHERE SUBSTRING(Name , 1 , 3) = @Name;
GO


--Create a Covering NONCLUSTERED INDEX   
CREATE NONCLUSTERED INDEX IX_Product_Name 
ON Production.Product(NAME ASC)
INCLUDE(ProductNumber, Color, StandardCost);
GO


--Our Index is being used however we are still seeing an index scan
--because of the SUBSTRING function in the predicate
DECLARE @Name nvarchar(50) = 'Mou';
SELECT Name
     , ProductNumber
     , Color
     , StandardCost
FROM Production.Product P
WHERE SUBSTRING(Name , 1 , 3) = @Name;
GO


-- Re-writing the query to use the LIKE operator get us the index seek that we desire
DECLARE @Name nvarchar(50) = 'Mou';
SELECT Name
     , ProductNumber
     , Color
     , StandardCost
FROM Production.Product P
WHERE Name LIKE @Name + '%';
GO



/*********************************************************************************** 
* PART III -  Optional Parameters
* Youre a DBA working on a query for a Reporting Services Report.  This report 
* shows product specific details and optionally if a product name is passed in, 
* only the matching products details will be listed.  Users are reporting that the 
* report is taking a long time to return results.  You determine that the issue 
* with the report is the following query.  You also notice that an existing 
* covering index is available but not being used
************************************************************************************/
DECLARE @Name nvarchar(50) = 'Flat Washer 2';
SELECT Name
     , ProductNumber
     , Color
     , StandardCost
FROM Production.Product P
WHERE COALESCE(@Name , Name) = Name;
GO


--By splitting the query into two separate queries with a UNION ALL
--allows us to use the index seek when the optional variable is not null
DECLARE @Name nvarchar(50) = 'Flat Washer 2';
SELECT Name
     , ProductNumber
     , Color
     , StandardCost
FROM Production.Product P
WHERE Name = @Name AND Name IS NOT NULL   
UNION ALL
SELECT Name
     , ProductNumber
     , Color
     , StandardCost
FROM Production.Product P
WHERE @Name IS NULL;
GO



/***********************************************************************************
* PART IV - Date Functions
* Youre a DBA working on a new query for an Inventory System.  
* You have been tasked with writing a query that will return all products 
* that will be used starting tomorrow through the next 30 days.  
* You notice that the query seems to take a long time.
************************************************************************************/
SELECT WorkOrderID
     , wo.ProductID
     , DueDate
FROM Production.WorkOrder wo
WHERE DATEDIFF(day , '20080601' , DueDate) > 0
  AND DATEDIFF(day , '20080601' , DueDate) <= 30;
GO


--Create a Covering NONCLUSTERED INDEX   
CREATE NONCLUSTERED INDEX IX_WorkOrder_DueDate 
ON Production.WorkOrder(DueDate)
INCLUDE(ProductID);
GO


--Our index is not used because the indexed column is in the DateDiff function
SELECT WorkOrderID
     , wo.ProductID
     , DueDate
FROM Production.WorkOrder wo
WHERE DATEDIFF(day , '20080601' , DueDate) > 0
  AND DATEDIFF(day , '20080601' , DueDate) <= 30;
GO


--Re-writing the query pulling the indexed column DueDate from the function, 
--we are now able to use the covering index
SELECT WorkOrderID
     , wo.ProductID
     , DueDate
FROM Production.WorkOrder wo
WHERE DueDate > DATEADD(day , 0 , '20080601')
  AND DueDate <= DATEADD(day , 30 , '20080601');
GO


/* END SARGable DEMO */