/***********************************************************************************
* T-SQL: Simple Changes That Go a Long Way
* Dave Valentine
* OUTPUT Demo SQL Script
***********************************************************************************/

USE AdventureWorks2012;
GO

--SET STATISTICS IO ON so we can see disk activity
SET STATISTICS IO ON;
GO



/***********************************************************************************
* PART I
*
* Using the OUTPUT statement we can capture the inserted ScrapReasonID Primary Key.
* We can store this primary key into a table, which we will see later.
***********************************************************************************/
INSERT INTO Production.ScrapReason
         (Name, ModifiedDate)
OUTPUT INSERTED.ScrapReasonID
VALUES   (N'TSQL Scrap', GETDATE());
GO



/***********************************************************************************
* Using the @@IDENITY we will capture the recently added identity and use it to
* delete the new record.  We can use the DELETED.* to show all columns which
* have been deleted.
***********************************************************************************/
DECLARE @ScrapReasonID smallint = @@IDENTITY;

DELETE FROM Production.ScrapReason
OUTPUT DELETED.*
WHERE ScrapReasonID = @ScrapReasonID;
GO



/***********************************************************************************
* Add the same TSQL Scrap record back into the system, but this time we want to 
* update the record and show both INSERTED and DELETED outputs.
***********************************************************************************/
INSERT INTO Production.ScrapReason
         (Name, ModifiedDate)
VALUES   (N'TSQL Scrap', GETDATE());
GO



/***********************************************************************************
* Get the identity of the most recently added record this time with SCOPE_IDENTITY.  
* Using the identity we can update and output both the inserted and deleted records
* from the update command.  Note the use of INSERTED and DELETED, but no UPDATED.
***********************************************************************************/
DECLARE @ScrapReasonID smallint = SCOPE_IDENTITY();

UPDATE Production.ScrapReason
SET Name = N'T-SQL Scrap'
OUTPUT DELETED.Name AS Before, INSERTED.Name AS After, INSERTED.ScrapReasonID
WHERE ScrapReasonID = @ScrapReasonID;
GO



/***********************************************************************************
* PART II
*
* Declare a Table Variable to store the values that will be outputted from
* UPDATE command.  This is especially helpful if you have multiple tables that need
* to be updated; this can be a replacement for a trigger.
***********************************************************************************/
DECLARE @ScrapReasonChanges TABLE (
   ScrapReasonID smallint
 , NameBefore Name
 , NameAfter Name
 , ModifiedDate datetime);

UPDATE   Production.ScrapReason
SET      ModifiedDate = GETDATE()
OUTPUT   INSERTED.ScrapReasonID
       , DELETED.Name AS NameBefore
       , INSERTED.Name AS NameAfter
       , INSERTED.ModifiedDate
INTO @ScrapReasonChanges (ScrapReasonID, NameBefore, NameAfter, ModifiedDate)

SELECT ScrapReasonID
     , NameBefore
     , NameAfter
     , ModifiedDate
FROM @ScrapReasonChanges SRC;
 
/* END OUTPUT DEMO */