/***********************************************************************************
* T-SQL: Simple Changes That Go a Long Way
* Dave Valentine
* Table Valued Parameter Demo SQL Script

ProductID	  LocationID  Shelf	  Bin Quantity
1		  1		    A	  1	 408
1		  6		    B	  5	 324
1		  50		    A	  5	 353
2		  1		    A	  2	 427
2		  6		    B	  1	 318
2		  50		    A	  6	 364
3		  1		    A	  7	 585
3		  6		    B	  9	 443
3		  50		    A	  10	 324
4		  1		    A	  6	 512
4		  6		    B	  10	 422
4		  50		    A	  11	 388
***********************************************************************************/

USE AdventureWorks2012;
GO


/***********************************************************************************
* Using the IN logical operator we can search or tables for a set of data.  This 
* operator is great especially if you know the values up front.
***********************************************************************************/
IF OBJECT_ID('Production.GetProductInventory') IS NOT NULL
    DROP PROCEDURE Production.GetProductInventory;
GO

CREATE PROCEDURE Production.GetProductInventory
AS
SELECT ProductID
     , LocationID
     , Shelf
     , Bin
     , Quantity
FROM Production.ProductInventory
WHERE ProductID IN (1, 2, 3, 4);
GO

-- Execute the procedure
EXEC Production.GetProductInventory;
GO



/***********************************************************************************
* So what happens if we want to pass a set of ID's into this procedure rather than
* hard coding them in the IN operator.  The following query throws a data 
* conversion error.
***********************************************************************************/
IF OBJECT_ID('Production.GetProductInventory') IS NOT NULL
    DROP PROCEDURE Production.GetProductInventory;
GO

CREATE PROCEDURE Production.GetProductInventory(@ProductIDs nvarchar(500))
AS
SELECT ProductID
     , LocationID
     , Shelf
     , Bin
     , Quantity
FROM production.ProductInventory
WHERE ProductID IN (@ProductIDs);
GO


-- Execute the procedure passing in the values through a nvarchar variable.
DECLARE @ProductIDs nvarchar(500) = N'1,2,3,4';
EXEC Production.GetProductInventory @ProductIDs;
GO



/***********************************************************************************
* We could store the whole query in a nvarchar(max) variable, then use 
* sp_executesql to execute a dynamic query.  Running the statements below returns the
* data we are looking for.  However, using dynamic sql could open the door for a 
* sql injection attack.
***********************************************************************************/
IF OBJECT_ID('Production.GetProductInventory') IS NOT NULL
    DROP PROCEDURE Production.GetProductInventory;
GO

CREATE PROCEDURE Production.GetProductInventory(@ProductIDs nvarchar(500))
AS
    DECLARE @ParamDef nvarchar(500) = N'@ProductIDs nvarchar(500)';
    DECLARE @ProductInventory nvarchar(max);
    SET @ProductInventory = N' SELECT ProductID, LocationID, Shelf, Bin, Quantity' + 
					    ' FROM Production.ProductInventory ' + 
					    ' WHERE ProductID IN (' + @ProductIDs + ');';
    EXECUTE sp_executesql @ProductInventory , @ParamDef , @ProductIDs;
GO


-- Execute the procedure passing in the values through a nvarchar variable.
DECLARE @ProductIDs nvarchar(500) = '1,2,3,4';
EXEC Production.GetProductInventory @ProductIDs;
GO



/***********************************************************************************
* A better approach would be to create and use a Table Valued Parameter.  We can 
* join directly to the table instead of using the IN operator.  Running the query 
* below yields the results we are looking for. 
* Notice the Table Scan in the Execution Plan?
***********************************************************************************/
CREATE TYPE ProductIDs AS TABLE (ProductID int);
GO


IF OBJECT_ID('Production.GetProductInventory') IS NOT NULL
    DROP PROCEDURE Production.GetProductInventory;
GO

CREATE PROCEDURE Production.GetProductInventory(@ProductIDs ProductIDs READONLY)
AS
SELECT prin.ProductID
     , LocationID
     , Shelf
     , Bin
     , Quantity
FROM production.ProductInventory prin
JOIN @ProductIDs pids ON prin.ProductID = pids.ProductID;
GO


-- Declare values for the table variable
DECLARE @ProductIDs ProductIDs;
INSERT INTO @ProductIDs
VALUES (1), (2), (3), (4);

-- Execute the procedure passing in the table variable.
EXEC Production.GetProductInventory @ProductIDs;
GO




/***********************************************************************************
* Running the following query limits the results by ProductID's and the LocationID
* as well.
***********************************************************************************/
IF OBJECT_ID('Production.GetProductInventory') IS NOT NULL
    DROP PROCEDURE Production.GetProductInventory;
GO

DROP TYPE ProductIDs;
GO

CREATE TYPE ProductIDs AS TABLE (ProductID int, LocationID int);
GO

CREATE PROCEDURE Production.GetProductInventory(@ProductIDs ProductIDs READONLY)
AS
SELECT prin.ProductID
     , prin.LocationID
     , Shelf
     , Bin
     , Quantity
FROM production.ProductInventory prin
     JOIN @ProductIDs pids ON prin.ProductID = pids.ProductID
                          AND prin.LocationID = pids.LocationID;
GO

-- Declare values for the table variable
DECLARE @ProductIDs ProductIDs;
INSERT INTO @ProductIDs
VALUES(1, 1), (2, 6), (3, 6), (4, 50);

-- Execute the procedure passing in the table variable.
EXEC Production.GetProductInventory @ProductIDs;
GO


/* END OUTPUT DEMO */